# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Reporting Service (0.1.27)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class RestapiBanAccountActionResponse(Model):
    """Restapi ban account action response (restapi.BanAccountActionResponse)

    Properties:
        duration: (duration) REQUIRED int

        reason: (reason) REQUIRED str

        skip_notif: (skipNotif) REQUIRED bool

        type_: (type) REQUIRED str

        comment: (comment) OPTIONAL str
    """

    # region fields

    duration: int  # REQUIRED
    reason: str  # REQUIRED
    skip_notif: bool  # REQUIRED
    type_: str  # REQUIRED
    comment: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_duration(self, value: int) -> RestapiBanAccountActionResponse:
        self.duration = value
        return self

    def with_reason(self, value: str) -> RestapiBanAccountActionResponse:
        self.reason = value
        return self

    def with_skip_notif(self, value: bool) -> RestapiBanAccountActionResponse:
        self.skip_notif = value
        return self

    def with_type(self, value: str) -> RestapiBanAccountActionResponse:
        self.type_ = value
        return self

    def with_comment(self, value: str) -> RestapiBanAccountActionResponse:
        self.comment = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "duration"):
            result["duration"] = int(self.duration)
        elif include_empty:
            result["duration"] = 0
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "skip_notif"):
            result["skipNotif"] = bool(self.skip_notif)
        elif include_empty:
            result["skipNotif"] = False
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "comment"):
            result["comment"] = str(self.comment)
        elif include_empty:
            result["comment"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        duration: int,
        reason: str,
        skip_notif: bool,
        type_: str,
        comment: Optional[str] = None,
        **kwargs,
    ) -> RestapiBanAccountActionResponse:
        instance = cls()
        instance.duration = duration
        instance.reason = reason
        instance.skip_notif = skip_notif
        instance.type_ = type_
        if comment is not None:
            instance.comment = comment
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RestapiBanAccountActionResponse:
        instance = cls()
        if not dict_:
            return instance
        if "duration" in dict_ and dict_["duration"] is not None:
            instance.duration = int(dict_["duration"])
        elif include_empty:
            instance.duration = 0
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "skipNotif" in dict_ and dict_["skipNotif"] is not None:
            instance.skip_notif = bool(dict_["skipNotif"])
        elif include_empty:
            instance.skip_notif = False
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "comment" in dict_ and dict_["comment"] is not None:
            instance.comment = str(dict_["comment"])
        elif include_empty:
            instance.comment = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, RestapiBanAccountActionResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[RestapiBanAccountActionResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        RestapiBanAccountActionResponse,
        List[RestapiBanAccountActionResponse],
        Dict[Any, RestapiBanAccountActionResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "duration": "duration",
            "reason": "reason",
            "skipNotif": "skip_notif",
            "type": "type_",
            "comment": "comment",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "duration": True,
            "reason": True,
            "skipNotif": True,
            "type": True,
            "comment": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Match Service V2 (2.5.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiPlayerData(Model):
    """Api player data (api.PlayerData)

    Properties:
        attributes: (attributes) OPTIONAL Dict[str, Any]

        party_id: (partyID) OPTIONAL str

        player_id: (playerID) OPTIONAL str
    """

    # region fields

    attributes: Dict[str, Any]  # OPTIONAL
    party_id: str  # OPTIONAL
    player_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_attributes(self, value: Dict[str, Any]) -> ApiPlayerData:
        self.attributes = value
        return self

    def with_party_id(self, value: str) -> ApiPlayerData:
        self.party_id = value
        return self

    def with_player_id(self, value: str) -> ApiPlayerData:
        self.player_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "attributes"):
            result["attributes"] = {str(k0): v0 for k0, v0 in self.attributes.items()}
        elif include_empty:
            result["attributes"] = {}
        if hasattr(self, "party_id"):
            result["partyID"] = str(self.party_id)
        elif include_empty:
            result["partyID"] = ""
        if hasattr(self, "player_id"):
            result["playerID"] = str(self.player_id)
        elif include_empty:
            result["playerID"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        attributes: Optional[Dict[str, Any]] = None,
        party_id: Optional[str] = None,
        player_id: Optional[str] = None,
        **kwargs,
    ) -> ApiPlayerData:
        instance = cls()
        if attributes is not None:
            instance.attributes = attributes
        if party_id is not None:
            instance.party_id = party_id
        if player_id is not None:
            instance.player_id = player_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiPlayerData:
        instance = cls()
        if not dict_:
            return instance
        if "attributes" in dict_ and dict_["attributes"] is not None:
            instance.attributes = {
                str(k0): v0 for k0, v0 in dict_["attributes"].items()
            }
        elif include_empty:
            instance.attributes = {}
        if "partyID" in dict_ and dict_["partyID"] is not None:
            instance.party_id = str(dict_["partyID"])
        elif include_empty:
            instance.party_id = ""
        if "playerID" in dict_ and dict_["playerID"] is not None:
            instance.player_id = str(dict_["playerID"])
        elif include_empty:
            instance.player_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiPlayerData]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiPlayerData]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApiPlayerData, List[ApiPlayerData], Dict[Any, ApiPlayerData]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "attributes": "attributes",
            "partyID": "party_id",
            "playerID": "player_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "attributes": False,
            "partyID": False,
            "playerID": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.30.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.platform_reward import PlatformReward
from ..models.revocation_result import RevocationResult
from ..models.revoke_result import RevokeResult


class StatusEnum(StrEnum):
    FULFILLED = "FULFILLED"
    REVOKED = "REVOKED"
    REVOKE_FAILED = "REVOKE_FAILED"


class DLCRecord(Model):
    """DLC record (DLCRecord)

    Properties:
        id_: (id) OPTIONAL str

        obtained_at: (obtainedAt) OPTIONAL str

        revocation_result: (revocationResult) OPTIONAL RevocationResult

        revoke_results: (revokeResults) OPTIONAL List[RevokeResult]

        revoked_at: (revokedAt) OPTIONAL str

        rewards: (rewards) OPTIONAL List[PlatformReward]

        sources: (sources) OPTIONAL List[str]

        status: (status) OPTIONAL Union[str, StatusEnum]

        transaction_id: (transactionId) OPTIONAL str

        version: (version) OPTIONAL int
    """

    # region fields

    id_: str  # OPTIONAL
    obtained_at: str  # OPTIONAL
    revocation_result: RevocationResult  # OPTIONAL
    revoke_results: List[RevokeResult]  # OPTIONAL
    revoked_at: str  # OPTIONAL
    rewards: List[PlatformReward]  # OPTIONAL
    sources: List[str]  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    transaction_id: str  # OPTIONAL
    version: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> DLCRecord:
        self.id_ = value
        return self

    def with_obtained_at(self, value: str) -> DLCRecord:
        self.obtained_at = value
        return self

    def with_revocation_result(self, value: RevocationResult) -> DLCRecord:
        self.revocation_result = value
        return self

    def with_revoke_results(self, value: List[RevokeResult]) -> DLCRecord:
        self.revoke_results = value
        return self

    def with_revoked_at(self, value: str) -> DLCRecord:
        self.revoked_at = value
        return self

    def with_rewards(self, value: List[PlatformReward]) -> DLCRecord:
        self.rewards = value
        return self

    def with_sources(self, value: List[str]) -> DLCRecord:
        self.sources = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> DLCRecord:
        self.status = value
        return self

    def with_transaction_id(self, value: str) -> DLCRecord:
        self.transaction_id = value
        return self

    def with_version(self, value: int) -> DLCRecord:
        self.version = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "obtained_at"):
            result["obtainedAt"] = str(self.obtained_at)
        elif include_empty:
            result["obtainedAt"] = ""
        if hasattr(self, "revocation_result"):
            result["revocationResult"] = self.revocation_result.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["revocationResult"] = RevocationResult()
        if hasattr(self, "revoke_results"):
            result["revokeResults"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.revoke_results
            ]
        elif include_empty:
            result["revokeResults"] = []
        if hasattr(self, "revoked_at"):
            result["revokedAt"] = str(self.revoked_at)
        elif include_empty:
            result["revokedAt"] = ""
        if hasattr(self, "rewards"):
            result["rewards"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.rewards
            ]
        elif include_empty:
            result["rewards"] = []
        if hasattr(self, "sources"):
            result["sources"] = [str(i0) for i0 in self.sources]
        elif include_empty:
            result["sources"] = []
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        if hasattr(self, "version"):
            result["version"] = int(self.version)
        elif include_empty:
            result["version"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: Optional[str] = None,
        obtained_at: Optional[str] = None,
        revocation_result: Optional[RevocationResult] = None,
        revoke_results: Optional[List[RevokeResult]] = None,
        revoked_at: Optional[str] = None,
        rewards: Optional[List[PlatformReward]] = None,
        sources: Optional[List[str]] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        transaction_id: Optional[str] = None,
        version: Optional[int] = None,
        **kwargs,
    ) -> DLCRecord:
        instance = cls()
        if id_ is not None:
            instance.id_ = id_
        if obtained_at is not None:
            instance.obtained_at = obtained_at
        if revocation_result is not None:
            instance.revocation_result = revocation_result
        if revoke_results is not None:
            instance.revoke_results = revoke_results
        if revoked_at is not None:
            instance.revoked_at = revoked_at
        if rewards is not None:
            instance.rewards = rewards
        if sources is not None:
            instance.sources = sources
        if status is not None:
            instance.status = status
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        if version is not None:
            instance.version = version
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> DLCRecord:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "obtainedAt" in dict_ and dict_["obtainedAt"] is not None:
            instance.obtained_at = str(dict_["obtainedAt"])
        elif include_empty:
            instance.obtained_at = ""
        if "revocationResult" in dict_ and dict_["revocationResult"] is not None:
            instance.revocation_result = RevocationResult.create_from_dict(
                dict_["revocationResult"], include_empty=include_empty
            )
        elif include_empty:
            instance.revocation_result = RevocationResult()
        if "revokeResults" in dict_ and dict_["revokeResults"] is not None:
            instance.revoke_results = [
                RevokeResult.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["revokeResults"]
            ]
        elif include_empty:
            instance.revoke_results = []
        if "revokedAt" in dict_ and dict_["revokedAt"] is not None:
            instance.revoked_at = str(dict_["revokedAt"])
        elif include_empty:
            instance.revoked_at = ""
        if "rewards" in dict_ and dict_["rewards"] is not None:
            instance.rewards = [
                PlatformReward.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["rewards"]
            ]
        elif include_empty:
            instance.rewards = []
        if "sources" in dict_ and dict_["sources"] is not None:
            instance.sources = [str(i0) for i0 in dict_["sources"]]
        elif include_empty:
            instance.sources = []
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        if "version" in dict_ and dict_["version"] is not None:
            instance.version = int(dict_["version"])
        elif include_empty:
            instance.version = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DLCRecord]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DLCRecord]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[DLCRecord, List[DLCRecord], Dict[Any, DLCRecord]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "obtainedAt": "obtained_at",
            "revocationResult": "revocation_result",
            "revokeResults": "revoke_results",
            "revokedAt": "revoked_at",
            "rewards": "rewards",
            "sources": "sources",
            "status": "status",
            "transactionId": "transaction_id",
            "version": "version",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": False,
            "obtainedAt": False,
            "revocationResult": False,
            "revokeResults": False,
            "revokedAt": False,
            "rewards": False,
            "sources": False,
            "status": False,
            "transactionId": False,
            "version": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["FULFILLED", "REVOKED", "REVOKE_FAILED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.30.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    AUTHORISED = "AUTHORISED"
    AUTHORISE_FAILED = "AUTHORISE_FAILED"
    CHARGEBACK = "CHARGEBACK"
    CHARGEBACK_REVERSED = "CHARGEBACK_REVERSED"
    CHARGED = "CHARGED"
    CHARGE_FAILED = "CHARGE_FAILED"
    DELETED = "DELETED"
    INIT = "INIT"
    NOTIFICATION_OF_CHARGEBACK = "NOTIFICATION_OF_CHARGEBACK"
    REFUNDED = "REFUNDED"
    REFUNDING = "REFUNDING"
    REFUND_FAILED = "REFUND_FAILED"
    REQUEST_FOR_INFORMATION = "REQUEST_FOR_INFORMATION"


class PaymentOrderRefundResult(Model):
    """Payment order refund result (PaymentOrderRefundResult)

    Properties:
        created_time: (createdTime) REQUIRED str

        namespace: (namespace) REQUIRED str

        payment_order_no: (paymentOrderNo) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        refunded_time: (refundedTime) OPTIONAL str

        target_namespace: (targetNamespace) OPTIONAL str

        target_user_id: (targetUserId) OPTIONAL str
    """

    # region fields

    created_time: str  # REQUIRED
    namespace: str  # REQUIRED
    payment_order_no: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    refunded_time: str  # OPTIONAL
    target_namespace: str  # OPTIONAL
    target_user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_time(self, value: str) -> PaymentOrderRefundResult:
        self.created_time = value
        return self

    def with_namespace(self, value: str) -> PaymentOrderRefundResult:
        self.namespace = value
        return self

    def with_payment_order_no(self, value: str) -> PaymentOrderRefundResult:
        self.payment_order_no = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> PaymentOrderRefundResult:
        self.status = value
        return self

    def with_refunded_time(self, value: str) -> PaymentOrderRefundResult:
        self.refunded_time = value
        return self

    def with_target_namespace(self, value: str) -> PaymentOrderRefundResult:
        self.target_namespace = value
        return self

    def with_target_user_id(self, value: str) -> PaymentOrderRefundResult:
        self.target_user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_time"):
            result["createdTime"] = str(self.created_time)
        elif include_empty:
            result["createdTime"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "payment_order_no"):
            result["paymentOrderNo"] = str(self.payment_order_no)
        elif include_empty:
            result["paymentOrderNo"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "refunded_time"):
            result["refundedTime"] = str(self.refunded_time)
        elif include_empty:
            result["refundedTime"] = ""
        if hasattr(self, "target_namespace"):
            result["targetNamespace"] = str(self.target_namespace)
        elif include_empty:
            result["targetNamespace"] = ""
        if hasattr(self, "target_user_id"):
            result["targetUserId"] = str(self.target_user_id)
        elif include_empty:
            result["targetUserId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_time: str,
        namespace: str,
        payment_order_no: str,
        status: Union[str, StatusEnum],
        refunded_time: Optional[str] = None,
        target_namespace: Optional[str] = None,
        target_user_id: Optional[str] = None,
        **kwargs,
    ) -> PaymentOrderRefundResult:
        instance = cls()
        instance.created_time = created_time
        instance.namespace = namespace
        instance.payment_order_no = payment_order_no
        instance.status = status
        if refunded_time is not None:
            instance.refunded_time = refunded_time
        if target_namespace is not None:
            instance.target_namespace = target_namespace
        if target_user_id is not None:
            instance.target_user_id = target_user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PaymentOrderRefundResult:
        instance = cls()
        if not dict_:
            return instance
        if "createdTime" in dict_ and dict_["createdTime"] is not None:
            instance.created_time = str(dict_["createdTime"])
        elif include_empty:
            instance.created_time = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "paymentOrderNo" in dict_ and dict_["paymentOrderNo"] is not None:
            instance.payment_order_no = str(dict_["paymentOrderNo"])
        elif include_empty:
            instance.payment_order_no = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "refundedTime" in dict_ and dict_["refundedTime"] is not None:
            instance.refunded_time = str(dict_["refundedTime"])
        elif include_empty:
            instance.refunded_time = ""
        if "targetNamespace" in dict_ and dict_["targetNamespace"] is not None:
            instance.target_namespace = str(dict_["targetNamespace"])
        elif include_empty:
            instance.target_namespace = ""
        if "targetUserId" in dict_ and dict_["targetUserId"] is not None:
            instance.target_user_id = str(dict_["targetUserId"])
        elif include_empty:
            instance.target_user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PaymentOrderRefundResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PaymentOrderRefundResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        PaymentOrderRefundResult,
        List[PaymentOrderRefundResult],
        Dict[Any, PaymentOrderRefundResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdTime": "created_time",
            "namespace": "namespace",
            "paymentOrderNo": "payment_order_no",
            "status": "status",
            "refundedTime": "refunded_time",
            "targetNamespace": "target_namespace",
            "targetUserId": "target_user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdTime": True,
            "namespace": True,
            "paymentOrderNo": True,
            "status": True,
            "refundedTime": False,
            "targetNamespace": False,
            "targetUserId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": [
                "AUTHORISED",
                "AUTHORISE_FAILED",
                "CHARGEBACK",
                "CHARGEBACK_REVERSED",
                "CHARGED",
                "CHARGE_FAILED",
                "DELETED",
                "INIT",
                "NOTIFICATION_OF_CHARGEBACK",
                "REFUNDED",
                "REFUNDING",
                "REFUND_FAILED",
                "REQUEST_FOR_INFORMATION",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import BasicCategoryInfo
from ..models import CategoryCreate
from ..models import CategoryInfo
from ..models import CategoryUpdate
from ..models import ErrorEntity
from ..models import FullCategoryInfo
from ..models import HierarchicalCategoryInfo
from ..models import ValidationErrorEntity

from ..operations.category import CreateCategory
from ..operations.category import DeleteCategory
from ..operations.category import DownloadCategories
from ..operations.category import GetCategory
from ..operations.category import GetChildCategories
from ..operations.category import GetDescendantCategories
from ..operations.category import GetRootCategories
from ..operations.category import ListCategoriesBasic
from ..operations.category import PublicGetCategory
from ..operations.category import PublicGetChildCategories
from ..operations.category import PublicGetDescendantCategories
from ..operations.category import PublicGetRootCategories
from ..operations.category import UpdateCategory


@same_doc_as(CreateCategory)
def create_category(
    store_id: str,
    body: Optional[CategoryCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create category (createCategory)

    This API is used to create category.

    A category is a path separated by "/". A category also has localized display names. Example:


        {

            "categoryPath": "/games",

            "localizationDisplayNames": {"en" : "Games"}

        }

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=1 (CREATE)
      *  Returns : created category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories

        method: POST

        tags: ["Category"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CategoryCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullCategoryInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}] | 30141: Store [{storeId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30271: Category [{categoryPath}] already exists in namespace [{namespace}] | 30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCategory.create(
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCategory)
async def create_category_async(
    store_id: str,
    body: Optional[CategoryCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create category (createCategory)

    This API is used to create category.

    A category is a path separated by "/". A category also has localized display names. Example:


        {

            "categoryPath": "/games",

            "localizationDisplayNames": {"en" : "Games"}

        }

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=1 (CREATE)
      *  Returns : created category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories

        method: POST

        tags: ["Category"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CategoryCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullCategoryInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}] | 30141: Store [{storeId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30271: Category [{categoryPath}] already exists in namespace [{namespace}] | 30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCategory.create(
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteCategory)
def delete_category(
    category_path: str,
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete category (deleteCategory)

    This API is used to delete category by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=8 (DELETE)
      *  Returns : the deleted category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [DELETE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}

        method: DELETE

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullCategoryInfo (successful operation)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30272: Category [{categoryPath}] is not empty in namespace [{namespace}] | 30173: Published store can't modify content)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteCategory.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteCategory)
async def delete_category_async(
    category_path: str,
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete category (deleteCategory)

    This API is used to delete category by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=8 (DELETE)
      *  Returns : the deleted category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [DELETE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}

        method: DELETE

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullCategoryInfo (successful operation)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30272: Category [{categoryPath}] is not empty in namespace [{namespace}] | 30173: Published store can't modify content)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteCategory.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DownloadCategories)
def download_categories(
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download store's structured categories (downloadCategories)

    This API is used to download store's structured categories.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store content)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store content)
      *  Returns : structured categories

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/download

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[HierarchicalCategoryInfo] (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DownloadCategories.create(
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DownloadCategories)
async def download_categories_async(
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download store's structured categories (downloadCategories)

    This API is used to download store's structured categories.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store content)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store content)
      *  Returns : structured categories

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/download

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[HierarchicalCategoryInfo] (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DownloadCategories.create(
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCategory)
def get_category(
    category_path: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get category (getCategory)

    This API is used to get category by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - FullCategoryInfo (successful operation)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCategory.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCategory)
async def get_category_async(
    category_path: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get category (getCategory)

    This API is used to get category by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - FullCategoryInfo (successful operation)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCategory.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetChildCategories)
def get_child_categories(
    category_path: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get child categories (getChildCategories)

    This API is used to get child categories by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : list of child categories data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}/children

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[FullCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetChildCategories.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetChildCategories)
async def get_child_categories_async(
    category_path: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get child categories (getChildCategories)

    This API is used to get child categories by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : list of child categories data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}/children

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[FullCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetChildCategories.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetDescendantCategories)
def get_descendant_categories(
    category_path: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get descendant categories (getDescendantCategories)

    This API is used to get descendant categories by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : list of descendant categories data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}/descendants

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[FullCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetDescendantCategories.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetDescendantCategories)
async def get_descendant_categories_async(
    category_path: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get descendant categories (getDescendantCategories)

    This API is used to get descendant categories by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : list of descendant categories data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}/descendants

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[FullCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetDescendantCategories.create(
        category_path=category_path,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetRootCategories)
def get_root_categories(
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get root categories (getRootCategories)

    This API is used to get root categories.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : root category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[FullCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetRootCategories.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetRootCategories)
async def get_root_categories_async(
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get root categories (getRootCategories)

    This API is used to get root categories.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : root category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[FullCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetRootCategories.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListCategoriesBasic)
def list_categories_basic(
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List categories' basic info (listCategoriesBasic)

    This API is used to list all categories' basic info of a store ordered by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : list of categories' paths

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/basic

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[BasicCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListCategoriesBasic.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListCategoriesBasic)
async def list_categories_basic_async(
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List categories' basic info (listCategoriesBasic)

    This API is used to list all categories' basic info of a store ordered by category path.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=2 (READ)
      *  Returns : list of categories' paths

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/basic

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[BasicCategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListCategoriesBasic.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetCategory)
def public_get_category(
    category_path: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Category (publicGetCategory)

    This API is used to get category by category path.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1 (CREATE)(user with this permission can view draft store category)
      *  Returns : category data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/{categoryPath}

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - CategoryInfo (successful operation)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetCategory.create(
        category_path=category_path,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetCategory)
async def public_get_category_async(
    category_path: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Category (publicGetCategory)

    This API is used to get category by category path.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1 (CREATE)(user with this permission can view draft store category)
      *  Returns : category data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/{categoryPath}

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - CategoryInfo (successful operation)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetCategory.create(
        category_path=category_path,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetChildCategories)
def public_get_child_categories(
    category_path: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Child Categories (publicGetChildCategories)

    This API is used to get child categories by category path.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store category)
      *  Returns : list of child categories data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/{categoryPath}/children

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[CategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetChildCategories.create(
        category_path=category_path,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetChildCategories)
async def public_get_child_categories_async(
    category_path: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Child Categories (publicGetChildCategories)

    This API is used to get child categories by category path.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store category)
      *  Returns : list of child categories data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/{categoryPath}/children

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[CategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetChildCategories.create(
        category_path=category_path,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetDescendantCategories)
def public_get_descendant_categories(
    category_path: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Descendant Categories (publicGetDescendantCategories)

    This API is used to get descendant categories by category path.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store category)
      *  Returns : list of descendant categories data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/{categoryPath}/descendants

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[CategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetDescendantCategories.create(
        category_path=category_path,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetDescendantCategories)
async def public_get_descendant_categories_async(
    category_path: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Descendant Categories (publicGetDescendantCategories)

    This API is used to get descendant categories by category path.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store category)
      *  Returns : list of descendant categories data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories/{categoryPath}/descendants

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[CategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetDescendantCategories.create(
        category_path=category_path,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetRootCategories)
def public_get_root_categories(
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Root Categories (publicGetRootCategories)

    This API is used to get root categories.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store category)
      *  Returns : root category data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[CategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetRootCategories.create(
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetRootCategories)
async def public_get_root_categories_async(
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Root Categories (publicGetRootCategories)

    This API is used to get root categories.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store category)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store category)
      *  Returns : root category data

    Properties:
        url: /platform/public/namespaces/{namespace}/categories

        method: GET

        tags: ["Category"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[CategoryInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetRootCategories.create(
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateCategory)
def update_category(
    category_path: str,
    store_id: str,
    body: Optional[CategoryUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update category (updateCategory)

    This API is used to update category.

    The category update data is a category object, example as:


        {

            "storeId": "store-id",

            "localizationDisplayNames": {"en" : "Games"}

        }

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=4 (UPDATE)
      *  Returns : the updated category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}

        method: PUT

        tags: ["Category"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CategoryUpdate in body

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullCategoryInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCategory.create(
        category_path=category_path,
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateCategory)
async def update_category_async(
    category_path: str,
    store_id: str,
    body: Optional[CategoryUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update category (updateCategory)

    This API is used to update category.

    The category update data is a category object, example as:


        {

            "storeId": "store-id",

            "localizationDisplayNames": {"en" : "Games"}

        }

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CATEGORY", action=4 (UPDATE)
      *  Returns : the updated category data

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CATEGORY [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/categories/{categoryPath}

        method: PUT

        tags: ["Category"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CategoryUpdate in body

        category_path: (categoryPath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullCategoryInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCategory.create(
        category_path=category_path,
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import RestapiActionApiRequest
from ..models import RestapiActionApiResponse
from ..models import RestapiActionListApiResponse
from ..models import RestapiErrorResponse
from ..models import RestapiExtensionCategoryApiRequest
from ..models import RestapiExtensionCategoryApiResponse
from ..models import RestapiExtensionCategoryListApiResponse

from ..operations.admin_extension_categories_and_auto_moderation_actions import (
    AdminCreateExtensionCategory,
)
from ..operations.admin_extension_categories_and_auto_moderation_actions import (
    AdminCreateModAction,
)
from ..operations.admin_extension_categories_and_auto_moderation_actions import (
    AdminFindActionList,
)
from ..operations.admin_extension_categories_and_auto_moderation_actions import (
    AdminFindExtensionCategoryList,
)
from ..operations.admin_extension_categories_and_auto_moderation_actions import (
    AdminFindExtensionCategoryListOrderEnum,
    AdminFindExtensionCategoryListSortByEnum,
)


@same_doc_as(AdminCreateExtensionCategory)
def admin_create_extension_category(
    body: RestapiExtensionCategoryApiRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Extension Category (adminCreateExtensionCategory)

    Required permission: ADMIN:REPORTINGCONFIG [CREATE]
    Create extension category data

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [CREATE]

    Properties:
        url: /reporting/v1/admin/extensionCategories

        method: POST

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RestapiExtensionCategoryApiRequest in body

    Responses:
        201: Created - RestapiExtensionCategoryApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminCreateExtensionCategory.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateExtensionCategory)
async def admin_create_extension_category_async(
    body: RestapiExtensionCategoryApiRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Extension Category (adminCreateExtensionCategory)

    Required permission: ADMIN:REPORTINGCONFIG [CREATE]
    Create extension category data

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [CREATE]

    Properties:
        url: /reporting/v1/admin/extensionCategories

        method: POST

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RestapiExtensionCategoryApiRequest in body

    Responses:
        201: Created - RestapiExtensionCategoryApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminCreateExtensionCategory.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminCreateModAction)
def admin_create_mod_action(
    body: RestapiActionApiRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Auto Moderation Action data (adminCreateModAction)

    Required permission: ADMIN:REPORTINGCONFIG [CREATE]
    Create auto moderation action

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [CREATE]

    Properties:
        url: /reporting/v1/admin/extensionActions

        method: POST

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RestapiActionApiRequest in body

    Responses:
        201: Created - RestapiActionApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminCreateModAction.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateModAction)
async def admin_create_mod_action_async(
    body: RestapiActionApiRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Auto Moderation Action data (adminCreateModAction)

    Required permission: ADMIN:REPORTINGCONFIG [CREATE]
    Create auto moderation action

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [CREATE]

    Properties:
        url: /reporting/v1/admin/extensionActions

        method: POST

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RestapiActionApiRequest in body

    Responses:
        201: Created - RestapiActionApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminCreateModAction.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminFindActionList)
def admin_find_action_list(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Find Auto Moderation Action List (adminFindActionList)

    Required permission: ADMIN:REPORTINGCONFIG [READ]
    Get a list of auto moderation actions

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [READ]

    Properties:
        url: /reporting/v1/admin/extensionActions

        method: GET

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - RestapiActionListApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminFindActionList.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminFindActionList)
async def admin_find_action_list_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Find Auto Moderation Action List (adminFindActionList)

    Required permission: ADMIN:REPORTINGCONFIG [READ]
    Get a list of auto moderation actions

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [READ]

    Properties:
        url: /reporting/v1/admin/extensionActions

        method: GET

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - RestapiActionListApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminFindActionList.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminFindExtensionCategoryList)
def admin_find_extension_category_list(
    order: Optional[Union[str, AdminFindExtensionCategoryListOrderEnum]] = None,
    sort_by: Optional[Union[str, AdminFindExtensionCategoryListSortByEnum]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Find Extension Category List (adminFindExtensionCategoryList)

    Required permission: ADMIN:REPORTINGCONFIG [READ]
    Get a list of extension category data with the specified name

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [READ]

    Properties:
        url: /reporting/v1/admin/extensionCategories

        method: GET

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        order: (order) OPTIONAL Union[str, OrderEnum] in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

    Responses:
        200: OK - RestapiExtensionCategoryListApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminFindExtensionCategoryList.create(
        order=order,
        sort_by=sort_by,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminFindExtensionCategoryList)
async def admin_find_extension_category_list_async(
    order: Optional[Union[str, AdminFindExtensionCategoryListOrderEnum]] = None,
    sort_by: Optional[Union[str, AdminFindExtensionCategoryListSortByEnum]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Find Extension Category List (adminFindExtensionCategoryList)

    Required permission: ADMIN:REPORTINGCONFIG [READ]
    Get a list of extension category data with the specified name

    Required Permission(s):
        - ADMIN:REPORTINGCONFIG [READ]

    Properties:
        url: /reporting/v1/admin/extensionCategories

        method: GET

        tags: ["Admin Extension Categories and Auto Moderation Actions"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        order: (order) OPTIONAL Union[str, OrderEnum] in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

    Responses:
        200: OK - RestapiExtensionCategoryListApiResponse

        400: Bad Request - RestapiErrorResponse

        500: Internal Server Error - RestapiErrorResponse
    """
    request = AdminFindExtensionCategoryList.create(
        order=order,
        sort_by=sort_by,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Lobby Server."""

__version__ = "3.22.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# admin
from .wrappers import create_template
from .wrappers import create_template_async
from .wrappers import delete_template_localization
from .wrappers import delete_template_localization_async
from .wrappers import delete_template_slug
from .wrappers import delete_template_slug_async
from .wrappers import free_form_notification
from .wrappers import free_form_notification_async
from .wrappers import get_game_template
from .wrappers import get_game_template_async
from .wrappers import get_localization_template
from .wrappers import get_localization_template_async
from .wrappers import get_slug_template
from .wrappers import get_slug_template_async
from .wrappers import notification_with_template
from .wrappers import notification_with_template_async
from .wrappers import publish_template
from .wrappers import publish_template_async
from .wrappers import update_localization_template
from .wrappers import update_localization_template_async

# config
from .wrappers import admin_export_config_v1
from .wrappers import admin_export_config_v1_async
from .wrappers import admin_get_all_config_v1
from .wrappers import admin_get_all_config_v1_async
from .wrappers import admin_get_config_v1
from .wrappers import admin_get_config_v1_async
from .wrappers import admin_import_config_v1
from .wrappers import admin_import_config_v1_async
from .wrappers import admin_update_config_v1
from .wrappers import admin_update_config_v1_async

# friends
from .wrappers import add_friends_without_confirmation
from .wrappers import add_friends_without_confirmation_async
from .wrappers import get_incoming_friend_requests
from .wrappers import get_incoming_friend_requests_async
from .wrappers import get_list_of_friends
from .wrappers import get_list_of_friends_async
from .wrappers import get_outgoing_friend_requests
from .wrappers import get_outgoing_friend_requests_async
from .wrappers import get_user_friends_updated
from .wrappers import get_user_friends_updated_async
from .wrappers import get_user_friends_with_platform
from .wrappers import get_user_friends_with_platform_async
from .wrappers import get_user_incoming_friends
from .wrappers import get_user_incoming_friends_async
from .wrappers import get_user_incoming_friends_with_time
from .wrappers import get_user_incoming_friends_with_time_async
from .wrappers import get_user_outgoing_friends
from .wrappers import get_user_outgoing_friends_async
from .wrappers import get_user_outgoing_friends_with_time
from .wrappers import get_user_outgoing_friends_with_time_async
from .wrappers import user_accept_friend_request
from .wrappers import user_accept_friend_request_async
from .wrappers import user_cancel_friend_request
from .wrappers import user_cancel_friend_request_async
from .wrappers import user_get_friendship_status
from .wrappers import user_get_friendship_status_async
from .wrappers import user_reject_friend_request
from .wrappers import user_reject_friend_request_async
from .wrappers import user_request_friend
from .wrappers import user_request_friend_async
from .wrappers import user_unfriend_request
from .wrappers import user_unfriend_request_async

# lobby_operations
from .wrappers import admin_join_party_v1
from .wrappers import admin_join_party_v1_async
from .wrappers import admin_update_party_attributes_v1
from .wrappers import admin_update_party_attributes_v1_async
from .wrappers import public_get_messages
from .wrappers import public_get_messages_async

# notification
from .wrappers import create_notification_template_v1_admin
from .wrappers import create_notification_template_v1_admin_async
from .wrappers import create_notification_topic_v1_admin
from .wrappers import create_notification_topic_v1_admin_async
from .wrappers import create_topic
from .wrappers import create_topic_async
from .wrappers import delete_notification_template_slug_v1_admin
from .wrappers import delete_notification_template_slug_v1_admin_async
from .wrappers import delete_notification_topic_v1_admin
from .wrappers import delete_notification_topic_v1_admin_async
from .wrappers import delete_template_localization_v1_admin
from .wrappers import delete_template_localization_v1_admin_async
from .wrappers import delete_topic_by_topic_name
from .wrappers import delete_topic_by_topic_name_async
from .wrappers import free_form_notification_by_user_id
from .wrappers import free_form_notification_by_user_id_async
from .wrappers import get_all_notification_templates_v1_admin
from .wrappers import get_all_notification_templates_v1_admin_async
from .wrappers import get_all_notification_topics_v1_admin
from .wrappers import get_all_notification_topics_v1_admin_async
from .wrappers import get_notification_topic_v1_admin
from .wrappers import get_notification_topic_v1_admin_async
from .wrappers import get_single_template_localization_v1_admin
from .wrappers import get_single_template_localization_v1_admin_async
from .wrappers import get_template_slug_localizations_template_v1_admin
from .wrappers import get_template_slug_localizations_template_v1_admin_async
from .wrappers import get_topic_by_namespace
from .wrappers import get_topic_by_namespace_async
from .wrappers import get_topic_by_topic_name
from .wrappers import get_topic_by_topic_name_async
from .wrappers import notification_with_template_by_user_id
from .wrappers import notification_with_template_by_user_id_async
from .wrappers import publish_template_localization_v1_admin
from .wrappers import publish_template_localization_v1_admin_async
from .wrappers import send_multiple_users_freeform_notification_v1_admin
from .wrappers import send_multiple_users_freeform_notification_v1_admin_async
from .wrappers import send_party_freeform_notification_v1_admin
from .wrappers import send_party_freeform_notification_v1_admin_async
from .wrappers import send_party_templated_notification_v1_admin
from .wrappers import send_party_templated_notification_v1_admin_async
from .wrappers import send_specific_user_freeform_notification_v1_admin
from .wrappers import send_specific_user_freeform_notification_v1_admin_async
from .wrappers import send_specific_user_templated_notification_v1_admin
from .wrappers import send_specific_user_templated_notification_v1_admin_async
from .wrappers import send_users_freeform_notification_v1_admin
from .wrappers import send_users_freeform_notification_v1_admin_async
from .wrappers import send_users_templated_notification_v1_admin
from .wrappers import send_users_templated_notification_v1_admin_async
from .wrappers import update_notification_topic_v1_admin
from .wrappers import update_notification_topic_v1_admin_async
from .wrappers import update_template_localization_v1_admin
from .wrappers import update_template_localization_v1_admin_async
from .wrappers import update_topic_by_topic_name
from .wrappers import update_topic_by_topic_name_async

# party
from .wrappers import admin_get_party_data_v1
from .wrappers import admin_get_party_data_v1_async
from .wrappers import admin_get_user_party_v1
from .wrappers import admin_get_user_party_v1_async
from .wrappers import public_get_party_data_v1
from .wrappers import public_get_party_data_v1_async
from .wrappers import public_set_party_limit_v1
from .wrappers import public_set_party_limit_v1_async
from .wrappers import public_update_party_attributes_v1
from .wrappers import public_update_party_attributes_v1_async

# player
from .wrappers import admin_bulk_block_players_v1
from .wrappers import admin_bulk_block_players_v1_async
from .wrappers import admin_get_all_player_session_attribute
from .wrappers import admin_get_all_player_session_attribute_async
from .wrappers import admin_get_bulk_player_blocked_players_v1
from .wrappers import admin_get_bulk_player_blocked_players_v1_async
from .wrappers import admin_get_lobby_ccu
from .wrappers import admin_get_lobby_ccu_async
from .wrappers import admin_get_player_blocked_by_players_v1
from .wrappers import admin_get_player_blocked_by_players_v1_async
from .wrappers import admin_get_player_blocked_players_v1
from .wrappers import admin_get_player_blocked_players_v1_async
from .wrappers import admin_get_player_session_attribute
from .wrappers import admin_get_player_session_attribute_async
from .wrappers import admin_set_player_session_attribute
from .wrappers import admin_set_player_session_attribute_async
from .wrappers import public_get_player_blocked_by_players_v1
from .wrappers import public_get_player_blocked_by_players_v1_async
from .wrappers import public_get_player_blocked_players_v1
from .wrappers import public_get_player_blocked_players_v1_async

# presence
from .wrappers import users_presence_handler_v1
from .wrappers import users_presence_handler_v1_async

# profanity
from .wrappers import admin_add_profanity_filter_into_list
from .wrappers import admin_add_profanity_filter_into_list_async
from .wrappers import admin_add_profanity_filters
from .wrappers import admin_add_profanity_filters_async
from .wrappers import admin_create_profanity_list
from .wrappers import admin_create_profanity_list_async
from .wrappers import admin_debug_profanity_filters
from .wrappers import admin_debug_profanity_filters_async
from .wrappers import admin_delete_profanity_filter
from .wrappers import admin_delete_profanity_filter_async
from .wrappers import admin_delete_profanity_list
from .wrappers import admin_delete_profanity_list_async
from .wrappers import admin_get_profanity_list_filters_v1
from .wrappers import admin_get_profanity_list_filters_v1_async
from .wrappers import admin_get_profanity_lists
from .wrappers import admin_get_profanity_lists_async
from .wrappers import admin_get_profanity_rule
from .wrappers import admin_get_profanity_rule_async
from .wrappers import admin_import_profanity_filters_from_file
from .wrappers import admin_import_profanity_filters_from_file_async
from .wrappers import admin_set_profanity_rule_for_namespace
from .wrappers import admin_set_profanity_rule_for_namespace_async
from .wrappers import admin_update_profanity_list
from .wrappers import admin_update_profanity_list_async
from .wrappers import admin_verify_message_profanity_response
from .wrappers import admin_verify_message_profanity_response_async

# third_party
from .wrappers import admin_create_third_party_config
from .wrappers import admin_create_third_party_config_async
from .wrappers import admin_delete_third_party_config
from .wrappers import admin_delete_third_party_config_async
from .wrappers import admin_get_third_party_config
from .wrappers import admin_get_third_party_config_async
from .wrappers import admin_update_third_party_config
from .wrappers import admin_update_third_party_config_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Fleet Command."""

__version__ = "0.1.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from ._account import account_create
from ._account import account_create_async
from ._account import account_get
from ._account import account_get_async
from ._account import account_link
from ._account import account_link_async
from ._account import account_link_token_get
from ._account import account_link_token_get_async

from ._ams_info import info_regions
from ._ams_info import info_regions_async
from ._ams_info import info_supported_instances
from ._ams_info import info_supported_instances_async

from ._fleets import fleet_claim_by_id
from ._fleets import fleet_claim_by_id_async
from ._fleets import fleet_claim_by_keys
from ._fleets import fleet_claim_by_keys_async
from ._fleets import fleet_create
from ._fleets import fleet_create_async
from ._fleets import fleet_delete
from ._fleets import fleet_delete_async
from ._fleets import fleet_get
from ._fleets import fleet_get_async
from ._fleets import fleet_list
from ._fleets import fleet_list_async
from ._fleets import fleet_servers
from ._fleets import fleet_servers_async
from ._fleets import fleet_update
from ._fleets import fleet_update_async

from ._images import image_get
from ._images import image_get_async
from ._images import image_list
from ._images import image_list_async
from ._images import image_patch
from ._images import image_patch_async

from ._operations import basic_health_check
from ._operations import basic_health_check_async
from ._operations import func1
from ._operations import func1_async

from ._servers import fleet_server_history
from ._servers import fleet_server_history_async
from ._servers import fleet_server_info
from ._servers import fleet_server_info_async
from ._servers import server_history
from ._servers import server_history_async

from ._watchdogs import watchdog_connect
from ._watchdogs import watchdog_connect_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Cloudsave Service (3.8.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.cloudsave.models import ModelsAdminConcurrentRecordRequest
from ..api.cloudsave.models import ModelsBulkGetGameRecordRequest
from ..api.cloudsave.models import ModelsBulkGetGameRecordResponse
from ..api.cloudsave.models import ModelsBulkGetPlayerRecordResponse
from ..api.cloudsave.models import ModelsBulkGetPlayerRecordSizeResponse
from ..api.cloudsave.models import ModelsBulkGetPlayerRecordsRequest
from ..api.cloudsave.models import ModelsBulkUserIDsRequest
from ..api.cloudsave.models import ModelsBulkUserKeyRequest
from ..api.cloudsave.models import ModelsConcurrentRecordRequest
from ..api.cloudsave.models import ModelsGameRecordRequest
from ..api.cloudsave.models import ModelsGameRecordResponse
from ..api.cloudsave.models import ModelsListGameRecordKeysResponse
from ..api.cloudsave.models import ModelsListPlayerRecordKeysResponse
from ..api.cloudsave.models import ModelsPagination
from ..api.cloudsave.models import ModelsPlayerRecordKeyInfo
from ..api.cloudsave.models import ModelsPlayerRecordRequest
from ..api.cloudsave.models import ModelsPlayerRecordResponse
from ..api.cloudsave.models import ModelsPlayerRecordSizeResponse
from ..api.cloudsave.models import ModelsResponseError
from ..api.cloudsave.models import ModelsUserKeyRequest


def create_models_admin_concurrent_record_request_example() -> (
    ModelsAdminConcurrentRecordRequest
):
    instance = ModelsAdminConcurrentRecordRequest()
    instance.set_by = randomize()
    instance.updated_at = randomize()
    instance.value = {randomize(): randomize()}
    return instance


def create_models_bulk_get_game_record_request_example() -> (
    ModelsBulkGetGameRecordRequest
):
    instance = ModelsBulkGetGameRecordRequest()
    instance.keys = [randomize()]
    return instance


def create_models_bulk_get_game_record_response_example() -> (
    ModelsBulkGetGameRecordResponse
):
    instance = ModelsBulkGetGameRecordResponse()
    instance.data = [create_models_game_record_response_example()]
    return instance


def create_models_bulk_get_player_record_response_example() -> (
    ModelsBulkGetPlayerRecordResponse
):
    instance = ModelsBulkGetPlayerRecordResponse()
    instance.data = [create_models_player_record_response_example()]
    return instance


def create_models_bulk_get_player_record_size_response_example() -> (
    ModelsBulkGetPlayerRecordSizeResponse
):
    instance = ModelsBulkGetPlayerRecordSizeResponse()
    instance.data = [create_models_player_record_size_response_example()]
    return instance


def create_models_bulk_get_player_records_request_example() -> (
    ModelsBulkGetPlayerRecordsRequest
):
    instance = ModelsBulkGetPlayerRecordsRequest()
    instance.keys = [randomize()]
    return instance


def create_models_bulk_user_i_ds_request_example() -> ModelsBulkUserIDsRequest:
    instance = ModelsBulkUserIDsRequest()
    instance.user_ids = [randomize()]
    return instance


def create_models_bulk_user_key_request_example() -> ModelsBulkUserKeyRequest:
    instance = ModelsBulkUserKeyRequest()
    instance.data = [create_models_user_key_request_example()]
    return instance


def create_models_concurrent_record_request_example() -> ModelsConcurrentRecordRequest:
    instance = ModelsConcurrentRecordRequest()
    instance.updated_at = randomize()
    instance.value = {randomize(): randomize()}
    return instance


def create_models_game_record_request_example() -> ModelsGameRecordRequest:
    instance = ModelsGameRecordRequest()
    return instance


def create_models_game_record_response_example() -> ModelsGameRecordResponse:
    instance = ModelsGameRecordResponse()
    instance.created_at = randomize("date")
    instance.key = randomize()
    instance.namespace = randomize("slug")
    instance.updated_at = randomize("date")
    instance.value = {randomize(): randomize()}
    instance.set_by = randomize()
    return instance


def create_models_list_game_record_keys_response_example() -> (
    ModelsListGameRecordKeysResponse
):
    instance = ModelsListGameRecordKeysResponse()
    instance.data = [randomize()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_list_player_record_keys_response_example() -> (
    ModelsListPlayerRecordKeysResponse
):
    instance = ModelsListPlayerRecordKeysResponse()
    instance.data = [create_models_player_record_key_info_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_pagination_example() -> ModelsPagination:
    instance = ModelsPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_player_record_key_info_example() -> ModelsPlayerRecordKeyInfo:
    instance = ModelsPlayerRecordKeyInfo()
    instance.key = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_player_record_request_example() -> ModelsPlayerRecordRequest:
    instance = ModelsPlayerRecordRequest()
    return instance


def create_models_player_record_response_example() -> ModelsPlayerRecordResponse:
    instance = ModelsPlayerRecordResponse()
    instance.created_at = randomize("date")
    instance.is_public = randomize("bool")
    instance.key = randomize()
    instance.namespace = randomize("slug")
    instance.updated_at = randomize("date")
    instance.user_id = randomize("uid")
    instance.value = {randomize(): randomize()}
    instance.set_by = randomize()
    return instance


def create_models_player_record_size_response_example() -> (
    ModelsPlayerRecordSizeResponse
):
    instance = ModelsPlayerRecordSizeResponse()
    instance.current_size = randomize("int", min_val=1, max_val=1000)
    instance.key = randomize()
    instance.namespace = randomize("slug")
    instance.remaining_size = randomize("int", min_val=1, max_val=1000)
    instance.user_id = randomize("uid")
    return instance


def create_models_response_error_example() -> ModelsResponseError:
    instance = ModelsResponseError()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance


def create_models_user_key_request_example() -> ModelsUserKeyRequest:
    instance = ModelsUserKeyRequest()
    instance.keys = [randomize()]
    instance.user_id = randomize("uid")
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Event Log Service."""

__version__ = "2.0.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .agent_type_description_handler import AgentTypeDescriptionHandler
from .event_id_description_handler import EventIDDescriptionHandler
from .event_level_description_9599c9 import EventLevelDescriptionHandler
from .event_type_description_handler import EventTypeDescriptionHandler
from .specific_agent_type_des_abc470 import SpecificAgentTypeDescriptionHandler
from .specific_event_id_descr_a3e19b import SpecificEventIDDescriptionHandler
from .specific_event_level_de_346d03 import SpecificEventLevelDescriptionHandler
from .specific_event_type_des_d1ea80 import SpecificEventTypeDescriptionHandler
from .specific_ux_description_86d7b4 import SpecificUXDescriptionHandler
from .ux_name_description_handler import UXNameDescriptionHandler



# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
import json
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import WebSocketMessage

from ....core import WebSocketMessageParserError

from ....core import WebSocketMessageParserException
from ....core import generate_websocket_message_id


class MessageNotif(WebSocketMessage):
    # region fields

    from_: str
    id_: str
    payload: str
    sent_at: int
    to: str
    topic: str

    # endregion fields

    # region methods

    # noinspection PyMethodMayBeStatic
    def to_wsm(self) -> str:
        # pylint: disable=no-self-use
        wsm = [f"type: {MessageNotif.get_type()}"]
        if hasattr(self, "from_") and self.from_:
            wsm.append(f"from: {self.from_}")
        id_ = self.id_ if hasattr(self, "id_") else generate_websocket_message_id()
        wsm.append(f"id: {id_}")
        if hasattr(self, "payload") and self.payload:
            wsm.append(f"payload: {self.payload}")
        if hasattr(self, "sent_at") and self.sent_at:
            wsm.append(f"sentAt: {self.sent_at}")
        if hasattr(self, "to") and self.to:
            wsm.append(f"to: {self.to}")
        if hasattr(self, "topic") and self.topic:
            wsm.append(f"topic: {self.topic}")
        return "\n".join(wsm)

    # endregion methods

    # region static methods

    @classmethod
    def create_from_wsm(cls, wsm: str, is_strict: bool = False) -> MessageNotif:
        instance = cls()
        if not wsm:
            return instance
        lines = wsm.splitlines(keepends=False)
        if len(lines) < 2:
            raise WebSocketMessageParserException(
                WebSocketMessageParserError.TypeFormatInvalid
            )
        id_line = lines[1]
        if not id_line.startswith("id: "):
            raise WebSocketMessageParserException(
                WebSocketMessageParserError.FieldFormatInvalid
            )
        instance.id_ = id_line.removeprefix("id: ")
        for line in lines[2:]:
            parts = line.split(":", 1)
            if len(parts) != 2:
                raise WebSocketMessageParserException(
                    WebSocketMessageParserError.FieldFormatInvalid
                )
            name, value = parts[0].strip(), parts[1].strip()
            if (not is_strict and name.casefold() == "from".casefold()) or (
                name == "from"
            ):
                instance.from_ = value
                continue
            if (not is_strict and name.casefold() == "payload".casefold()) or (
                name == "payload"
            ):
                instance.payload = value
                continue
            if (not is_strict and name.casefold() == "sentAt".casefold()) or (
                name == "sentAt"
            ):
                instance.sent_at = value
                continue
            if (not is_strict and name.casefold() == "to".casefold()) or (name == "to"):
                instance.to = value
                continue
            if (not is_strict and name.casefold() == "topic".casefold()) or (
                name == "topic"
            ):
                instance.topic = value
                continue
            if is_strict:
                raise WebSocketMessageParserException(
                    WebSocketMessageParserError.FieldTypeNotSupported
                )
        return instance

    @staticmethod
    def get_type() -> str:
        return "messageNotif"

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "from": "from_",
            "id": "id_",
            "payload": "payload",
            "sentAt": "sent_at",
            "to": "to",
            "topic": "topic",
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service (2.15.4)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_team import ModelsTeam


class ApimodelsUpdateGameSessionRequest(Model):
    """Apimodels update game session request (apimodels.UpdateGameSessionRequest)

    Properties:
        attributes: (attributes) REQUIRED Dict[str, Any]

        backfill_ticket_id: (backfillTicketID) REQUIRED str

        client_version: (clientVersion) REQUIRED str

        deployment: (deployment) REQUIRED str

        inactive_timeout: (inactiveTimeout) REQUIRED int

        invite_timeout: (inviteTimeout) REQUIRED int

        joinability: (joinability) REQUIRED str

        match_pool: (matchPool) REQUIRED str

        max_players: (maxPlayers) REQUIRED int

        min_players: (minPlayers) REQUIRED int

        requested_regions: (requestedRegions) REQUIRED List[str]

        teams: (teams) REQUIRED List[ModelsTeam]

        ticket_i_ds: (ticketIDs) REQUIRED List[str]

        type_: (type) REQUIRED str

        version: (version) REQUIRED int

        fallback_claim_keys: (fallbackClaimKeys) OPTIONAL List[str]

        preferred_claim_keys: (preferredClaimKeys) OPTIONAL List[str]
    """

    # region fields

    attributes: Dict[str, Any]  # REQUIRED
    backfill_ticket_id: str  # REQUIRED
    client_version: str  # REQUIRED
    deployment: str  # REQUIRED
    inactive_timeout: int  # REQUIRED
    invite_timeout: int  # REQUIRED
    joinability: str  # REQUIRED
    match_pool: str  # REQUIRED
    max_players: int  # REQUIRED
    min_players: int  # REQUIRED
    requested_regions: List[str]  # REQUIRED
    teams: List[ModelsTeam]  # REQUIRED
    ticket_i_ds: List[str]  # REQUIRED
    type_: str  # REQUIRED
    version: int  # REQUIRED
    fallback_claim_keys: List[str]  # OPTIONAL
    preferred_claim_keys: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_attributes(
        self, value: Dict[str, Any]
    ) -> ApimodelsUpdateGameSessionRequest:
        self.attributes = value
        return self

    def with_backfill_ticket_id(self, value: str) -> ApimodelsUpdateGameSessionRequest:
        self.backfill_ticket_id = value
        return self

    def with_client_version(self, value: str) -> ApimodelsUpdateGameSessionRequest:
        self.client_version = value
        return self

    def with_deployment(self, value: str) -> ApimodelsUpdateGameSessionRequest:
        self.deployment = value
        return self

    def with_inactive_timeout(self, value: int) -> ApimodelsUpdateGameSessionRequest:
        self.inactive_timeout = value
        return self

    def with_invite_timeout(self, value: int) -> ApimodelsUpdateGameSessionRequest:
        self.invite_timeout = value
        return self

    def with_joinability(self, value: str) -> ApimodelsUpdateGameSessionRequest:
        self.joinability = value
        return self

    def with_match_pool(self, value: str) -> ApimodelsUpdateGameSessionRequest:
        self.match_pool = value
        return self

    def with_max_players(self, value: int) -> ApimodelsUpdateGameSessionRequest:
        self.max_players = value
        return self

    def with_min_players(self, value: int) -> ApimodelsUpdateGameSessionRequest:
        self.min_players = value
        return self

    def with_requested_regions(
        self, value: List[str]
    ) -> ApimodelsUpdateGameSessionRequest:
        self.requested_regions = value
        return self

    def with_teams(self, value: List[ModelsTeam]) -> ApimodelsUpdateGameSessionRequest:
        self.teams = value
        return self

    def with_ticket_i_ds(self, value: List[str]) -> ApimodelsUpdateGameSessionRequest:
        self.ticket_i_ds = value
        return self

    def with_type(self, value: str) -> ApimodelsUpdateGameSessionRequest:
        self.type_ = value
        return self

    def with_version(self, value: int) -> ApimodelsUpdateGameSessionRequest:
        self.version = value
        return self

    def with_fallback_claim_keys(
        self, value: List[str]
    ) -> ApimodelsUpdateGameSessionRequest:
        self.fallback_claim_keys = value
        return self

    def with_preferred_claim_keys(
        self, value: List[str]
    ) -> ApimodelsUpdateGameSessionRequest:
        self.preferred_claim_keys = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "attributes"):
            result["attributes"] = {str(k0): v0 for k0, v0 in self.attributes.items()}
        elif include_empty:
            result["attributes"] = {}
        if hasattr(self, "backfill_ticket_id"):
            result["backfillTicketID"] = str(self.backfill_ticket_id)
        elif include_empty:
            result["backfillTicketID"] = ""
        if hasattr(self, "client_version"):
            result["clientVersion"] = str(self.client_version)
        elif include_empty:
            result["clientVersion"] = ""
        if hasattr(self, "deployment"):
            result["deployment"] = str(self.deployment)
        elif include_empty:
            result["deployment"] = ""
        if hasattr(self, "inactive_timeout"):
            result["inactiveTimeout"] = int(self.inactive_timeout)
        elif include_empty:
            result["inactiveTimeout"] = 0
        if hasattr(self, "invite_timeout"):
            result["inviteTimeout"] = int(self.invite_timeout)
        elif include_empty:
            result["inviteTimeout"] = 0
        if hasattr(self, "joinability"):
            result["joinability"] = str(self.joinability)
        elif include_empty:
            result["joinability"] = ""
        if hasattr(self, "match_pool"):
            result["matchPool"] = str(self.match_pool)
        elif include_empty:
            result["matchPool"] = ""
        if hasattr(self, "max_players"):
            result["maxPlayers"] = int(self.max_players)
        elif include_empty:
            result["maxPlayers"] = 0
        if hasattr(self, "min_players"):
            result["minPlayers"] = int(self.min_players)
        elif include_empty:
            result["minPlayers"] = 0
        if hasattr(self, "requested_regions"):
            result["requestedRegions"] = [str(i0) for i0 in self.requested_regions]
        elif include_empty:
            result["requestedRegions"] = []
        if hasattr(self, "teams"):
            result["teams"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.teams
            ]
        elif include_empty:
            result["teams"] = []
        if hasattr(self, "ticket_i_ds"):
            result["ticketIDs"] = [str(i0) for i0 in self.ticket_i_ds]
        elif include_empty:
            result["ticketIDs"] = []
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "version"):
            result["version"] = int(self.version)
        elif include_empty:
            result["version"] = 0
        if hasattr(self, "fallback_claim_keys"):
            result["fallbackClaimKeys"] = [str(i0) for i0 in self.fallback_claim_keys]
        elif include_empty:
            result["fallbackClaimKeys"] = []
        if hasattr(self, "preferred_claim_keys"):
            result["preferredClaimKeys"] = [str(i0) for i0 in self.preferred_claim_keys]
        elif include_empty:
            result["preferredClaimKeys"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        attributes: Dict[str, Any],
        backfill_ticket_id: str,
        client_version: str,
        deployment: str,
        inactive_timeout: int,
        invite_timeout: int,
        joinability: str,
        match_pool: str,
        max_players: int,
        min_players: int,
        requested_regions: List[str],
        teams: List[ModelsTeam],
        ticket_i_ds: List[str],
        type_: str,
        version: int,
        fallback_claim_keys: Optional[List[str]] = None,
        preferred_claim_keys: Optional[List[str]] = None,
        **kwargs,
    ) -> ApimodelsUpdateGameSessionRequest:
        instance = cls()
        instance.attributes = attributes
        instance.backfill_ticket_id = backfill_ticket_id
        instance.client_version = client_version
        instance.deployment = deployment
        instance.inactive_timeout = inactive_timeout
        instance.invite_timeout = invite_timeout
        instance.joinability = joinability
        instance.match_pool = match_pool
        instance.max_players = max_players
        instance.min_players = min_players
        instance.requested_regions = requested_regions
        instance.teams = teams
        instance.ticket_i_ds = ticket_i_ds
        instance.type_ = type_
        instance.version = version
        if fallback_claim_keys is not None:
            instance.fallback_claim_keys = fallback_claim_keys
        if preferred_claim_keys is not None:
            instance.preferred_claim_keys = preferred_claim_keys
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsUpdateGameSessionRequest:
        instance = cls()
        if not dict_:
            return instance
        if "attributes" in dict_ and dict_["attributes"] is not None:
            instance.attributes = {
                str(k0): v0 for k0, v0 in dict_["attributes"].items()
            }
        elif include_empty:
            instance.attributes = {}
        if "backfillTicketID" in dict_ and dict_["backfillTicketID"] is not None:
            instance.backfill_ticket_id = str(dict_["backfillTicketID"])
        elif include_empty:
            instance.backfill_ticket_id = ""
        if "clientVersion" in dict_ and dict_["clientVersion"] is not None:
            instance.client_version = str(dict_["clientVersion"])
        elif include_empty:
            instance.client_version = ""
        if "deployment" in dict_ and dict_["deployment"] is not None:
            instance.deployment = str(dict_["deployment"])
        elif include_empty:
            instance.deployment = ""
        if "inactiveTimeout" in dict_ and dict_["inactiveTimeout"] is not None:
            instance.inactive_timeout = int(dict_["inactiveTimeout"])
        elif include_empty:
            instance.inactive_timeout = 0
        if "inviteTimeout" in dict_ and dict_["inviteTimeout"] is not None:
            instance.invite_timeout = int(dict_["inviteTimeout"])
        elif include_empty:
            instance.invite_timeout = 0
        if "joinability" in dict_ and dict_["joinability"] is not None:
            instance.joinability = str(dict_["joinability"])
        elif include_empty:
            instance.joinability = ""
        if "matchPool" in dict_ and dict_["matchPool"] is not None:
            instance.match_pool = str(dict_["matchPool"])
        elif include_empty:
            instance.match_pool = ""
        if "maxPlayers" in dict_ and dict_["maxPlayers"] is not None:
            instance.max_players = int(dict_["maxPlayers"])
        elif include_empty:
            instance.max_players = 0
        if "minPlayers" in dict_ and dict_["minPlayers"] is not None:
            instance.min_players = int(dict_["minPlayers"])
        elif include_empty:
            instance.min_players = 0
        if "requestedRegions" in dict_ and dict_["requestedRegions"] is not None:
            instance.requested_regions = [str(i0) for i0 in dict_["requestedRegions"]]
        elif include_empty:
            instance.requested_regions = []
        if "teams" in dict_ and dict_["teams"] is not None:
            instance.teams = [
                ModelsTeam.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["teams"]
            ]
        elif include_empty:
            instance.teams = []
        if "ticketIDs" in dict_ and dict_["ticketIDs"] is not None:
            instance.ticket_i_ds = [str(i0) for i0 in dict_["ticketIDs"]]
        elif include_empty:
            instance.ticket_i_ds = []
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "version" in dict_ and dict_["version"] is not None:
            instance.version = int(dict_["version"])
        elif include_empty:
            instance.version = 0
        if "fallbackClaimKeys" in dict_ and dict_["fallbackClaimKeys"] is not None:
            instance.fallback_claim_keys = [
                str(i0) for i0 in dict_["fallbackClaimKeys"]
            ]
        elif include_empty:
            instance.fallback_claim_keys = []
        if "preferredClaimKeys" in dict_ and dict_["preferredClaimKeys"] is not None:
            instance.preferred_claim_keys = [
                str(i0) for i0 in dict_["preferredClaimKeys"]
            ]
        elif include_empty:
            instance.preferred_claim_keys = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsUpdateGameSessionRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsUpdateGameSessionRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsUpdateGameSessionRequest,
        List[ApimodelsUpdateGameSessionRequest],
        Dict[Any, ApimodelsUpdateGameSessionRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "attributes": "attributes",
            "backfillTicketID": "backfill_ticket_id",
            "clientVersion": "client_version",
            "deployment": "deployment",
            "inactiveTimeout": "inactive_timeout",
            "inviteTimeout": "invite_timeout",
            "joinability": "joinability",
            "matchPool": "match_pool",
            "maxPlayers": "max_players",
            "minPlayers": "min_players",
            "requestedRegions": "requested_regions",
            "teams": "teams",
            "ticketIDs": "ticket_i_ds",
            "type": "type_",
            "version": "version",
            "fallbackClaimKeys": "fallback_claim_keys",
            "preferredClaimKeys": "preferred_claim_keys",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "attributes": True,
            "backfillTicketID": True,
            "clientVersion": True,
            "deployment": True,
            "inactiveTimeout": True,
            "inviteTimeout": True,
            "joinability": True,
            "matchPool": True,
            "maxPlayers": True,
            "minPlayers": True,
            "requestedRegions": True,
            "teams": True,
            "ticketIDs": True,
            "type": True,
            "version": True,
            "fallbackClaimKeys": False,
            "preferredClaimKeys": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.30.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.client_request_parameter import ClientRequestParameter
from ..models.consume_item import ConsumeItem
from ..models.request_history import RequestHistory


class IapTypeEnum(StrEnum):
    APPLE = "APPLE"
    EPICGAMES = "EPICGAMES"
    GOOGLE = "GOOGLE"
    PLAYSTATION = "PLAYSTATION"
    STADIA = "STADIA"
    STEAM = "STEAM"
    TWITCH = "TWITCH"
    XBOX = "XBOX"


class StatusEnum(StrEnum):
    FAIL = "FAIL"
    PENDING = "PENDING"
    SUCCESS = "SUCCESS"


class IAPConsumeHistoryInfo(Model):
    """IAP consume history info (IAPConsumeHistoryInfo)

    Properties:
        client_request_parameter: (clientRequestParameter) OPTIONAL ClientRequestParameter

        consume_items: (consumeItems) OPTIONAL List[ConsumeItem]

        iap_type: (iapType) OPTIONAL Union[str, IapTypeEnum]

        id_: (id) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        request_body: (requestBody) OPTIONAL Dict[str, Any]

        request_histories: (requestHistories) OPTIONAL List[RequestHistory]

        request_url: (requestUrl) OPTIONAL str

        status: (status) OPTIONAL Union[str, StatusEnum]

        user_id: (userId) OPTIONAL str
    """

    # region fields

    client_request_parameter: ClientRequestParameter  # OPTIONAL
    consume_items: List[ConsumeItem]  # OPTIONAL
    iap_type: Union[str, IapTypeEnum]  # OPTIONAL
    id_: str  # OPTIONAL
    namespace: str  # OPTIONAL
    request_body: Dict[str, Any]  # OPTIONAL
    request_histories: List[RequestHistory]  # OPTIONAL
    request_url: str  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_client_request_parameter(
        self, value: ClientRequestParameter
    ) -> IAPConsumeHistoryInfo:
        self.client_request_parameter = value
        return self

    def with_consume_items(self, value: List[ConsumeItem]) -> IAPConsumeHistoryInfo:
        self.consume_items = value
        return self

    def with_iap_type(self, value: Union[str, IapTypeEnum]) -> IAPConsumeHistoryInfo:
        self.iap_type = value
        return self

    def with_id(self, value: str) -> IAPConsumeHistoryInfo:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> IAPConsumeHistoryInfo:
        self.namespace = value
        return self

    def with_request_body(self, value: Dict[str, Any]) -> IAPConsumeHistoryInfo:
        self.request_body = value
        return self

    def with_request_histories(
        self, value: List[RequestHistory]
    ) -> IAPConsumeHistoryInfo:
        self.request_histories = value
        return self

    def with_request_url(self, value: str) -> IAPConsumeHistoryInfo:
        self.request_url = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> IAPConsumeHistoryInfo:
        self.status = value
        return self

    def with_user_id(self, value: str) -> IAPConsumeHistoryInfo:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "client_request_parameter"):
            result["clientRequestParameter"] = self.client_request_parameter.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["clientRequestParameter"] = ClientRequestParameter()
        if hasattr(self, "consume_items"):
            result["consumeItems"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.consume_items
            ]
        elif include_empty:
            result["consumeItems"] = []
        if hasattr(self, "iap_type"):
            result["iapType"] = str(self.iap_type)
        elif include_empty:
            result["iapType"] = Union[str, IapTypeEnum]()
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "request_body"):
            result["requestBody"] = {
                str(k0): v0 for k0, v0 in self.request_body.items()
            }
        elif include_empty:
            result["requestBody"] = {}
        if hasattr(self, "request_histories"):
            result["requestHistories"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.request_histories
            ]
        elif include_empty:
            result["requestHistories"] = []
        if hasattr(self, "request_url"):
            result["requestUrl"] = str(self.request_url)
        elif include_empty:
            result["requestUrl"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_request_parameter: Optional[ClientRequestParameter] = None,
        consume_items: Optional[List[ConsumeItem]] = None,
        iap_type: Optional[Union[str, IapTypeEnum]] = None,
        id_: Optional[str] = None,
        namespace: Optional[str] = None,
        request_body: Optional[Dict[str, Any]] = None,
        request_histories: Optional[List[RequestHistory]] = None,
        request_url: Optional[str] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> IAPConsumeHistoryInfo:
        instance = cls()
        if client_request_parameter is not None:
            instance.client_request_parameter = client_request_parameter
        if consume_items is not None:
            instance.consume_items = consume_items
        if iap_type is not None:
            instance.iap_type = iap_type
        if id_ is not None:
            instance.id_ = id_
        if namespace is not None:
            instance.namespace = namespace
        if request_body is not None:
            instance.request_body = request_body
        if request_histories is not None:
            instance.request_histories = request_histories
        if request_url is not None:
            instance.request_url = request_url
        if status is not None:
            instance.status = status
        if user_id is not None:
            instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> IAPConsumeHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if (
            "clientRequestParameter" in dict_
            and dict_["clientRequestParameter"] is not None
        ):
            instance.client_request_parameter = ClientRequestParameter.create_from_dict(
                dict_["clientRequestParameter"], include_empty=include_empty
            )
        elif include_empty:
            instance.client_request_parameter = ClientRequestParameter()
        if "consumeItems" in dict_ and dict_["consumeItems"] is not None:
            instance.consume_items = [
                ConsumeItem.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["consumeItems"]
            ]
        elif include_empty:
            instance.consume_items = []
        if "iapType" in dict_ and dict_["iapType"] is not None:
            instance.iap_type = str(dict_["iapType"])
        elif include_empty:
            instance.iap_type = Union[str, IapTypeEnum]()
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "requestBody" in dict_ and dict_["requestBody"] is not None:
            instance.request_body = {
                str(k0): v0 for k0, v0 in dict_["requestBody"].items()
            }
        elif include_empty:
            instance.request_body = {}
        if "requestHistories" in dict_ and dict_["requestHistories"] is not None:
            instance.request_histories = [
                RequestHistory.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["requestHistories"]
            ]
        elif include_empty:
            instance.request_histories = []
        if "requestUrl" in dict_ and dict_["requestUrl"] is not None:
            instance.request_url = str(dict_["requestUrl"])
        elif include_empty:
            instance.request_url = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, IAPConsumeHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[IAPConsumeHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        IAPConsumeHistoryInfo,
        List[IAPConsumeHistoryInfo],
        Dict[Any, IAPConsumeHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "clientRequestParameter": "client_request_parameter",
            "consumeItems": "consume_items",
            "iapType": "iap_type",
            "id": "id_",
            "namespace": "namespace",
            "requestBody": "request_body",
            "requestHistories": "request_histories",
            "requestUrl": "request_url",
            "status": "status",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "clientRequestParameter": False,
            "consumeItems": False,
            "iapType": False,
            "id": False,
            "namespace": False,
            "requestBody": False,
            "requestHistories": False,
            "requestUrl": False,
            "status": False,
            "userId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "iapType": [
                "APPLE",
                "EPICGAMES",
                "GOOGLE",
                "PLAYSTATION",
                "STADIA",
                "STEAM",
                "TWITCH",
                "XBOX",
            ],
            "status": ["FAIL", "PENDING", "SUCCESS"],
        }

    # endregion static methods
